==> spread-5.2.0.24-py3-none-any.whl: spread_core/protocols/dali/driver/dali_interface.py <==
import logging
import threading

import can

from spread_core.tools.base_interface import AbstractInterfaceClass
from .. import frame
from ..command import Command, YesNoResponse, from_frame
from ..frame import BackwardModuleError
from ..gear import general as gear
from ..device import general as device
from ..address import Broadcast, BroadcastUnaddressed

_can_logger = logging.getLogger('can.interfaces.socketcan.socketcan')
_can_logger.level = logging.WARNING
_can_logger.getChild("rx").level = logging.WARNING

DOUBLE_SEND_FLAG = 'D'
THREE_BYTES_FLAG = 'F'
FORCE_ANSWER_FLAG = 'W'
DALI_ERROR_FLAG = 'E'


class DaliInterface(AbstractInterfaceClass):
    _response = None
    _request_event = threading.Event()
    _last_command = None
    last_external = None
    last_enabled_type = 0
    _dtr = [[0, 0, 0], [0, 0, 0]]
    _quiescent_mode = False

    def dtr(self, command, dtr_id):
        dali_index = 0 if isinstance(command, gear._GearCommand) else 1
        return self._dtr[dali_index][dtr_id]

    def send(self, *args, **kwargs):
        raise NotImplementedError()

    def on_dump(self, response, *args, **kwargs):
        if DALI_ERROR_FLAG in kwargs.get('flags', []):
            logging.debug('CAN-error received')
            return

        command = self._last_command
        if len(response) == 3 and response[0] & 1 == 0:
            self.on_dali2_event(response)
        elif command and response == command.frame.pack:
            command.on_commit()
            logging.debug('  -> ✓: {}'.format(str_bytes(response)))
            if command.is_commited and not command.is_query:
                self.release_condition()
        elif command and len(response) <= 1:
            command.on_response()
            try:
                if len(response) == 0:
                    if command.response == YesNoResponse:
                        result = command.response(None)
                    else:
                        result = command.response(frame.BackwardModuleError())
                else:
                    result = command.response(frame.BackwardFrame(response))

                self._response = result

                _flags_str = 'flags: [{}]'.format(', '.join(kwargs['flags'])) if kwargs.get('flags', []) else ''
                logging.debug('    <- {}'.format('; '.join([str(result), _flags_str])))
            except BaseException as ex:
                logging.error('[{}]: {}'.format(response, str(ex)))
            finally:
                self.release_condition()
        else:
            self.on_unknown_pack(response, *args, **kwargs)

    def on_unknown_pack(self, pack, *args, **kwargs):
        command = self.last_external
        if command and pack == command.frame.pack:
            command.on_commit()
            logging.debug('  >> ✓: {}'.format(str_bytes(pack)))
            if command.is_commited and not command.is_query:
                self.set_external(command)
        elif command and len(pack) <= 1:
            command.on_response()
            try:
                if 'E' in kwargs.get('flags', []):
                    result = command.response(frame.BackwardModuleError())
                elif len(pack) == 0:
                    if command.response == YesNoResponse:
                        result = command.response(None)
                    else:
                        result = command.response(frame.BackwardModuleError())
                else:
                    result = command.response(frame.BackwardFrame(pack))

                logging.debug('    << {}'.format('; '.join([str(result)])))

                args = []
                if isinstance(command, (gear._StandardCommand, gear._SpecialCommand)) and command._hasparam:
                    args.append(command.param)
                if command._uses_dtr0:
                    args.append(self.dtr(command, 0))
                if command._uses_dtr1:
                    args.append(self.dtr(command, 1))
                if command._uses_dtr2:
                    args.append(self.dtr(command, 2))

                self.set_external(command, *args, response=result)
            except BaseException as ex:
                logging.error('[{}]: {}'.format(pack, str(ex)))
            finally:
                self.set_external(None)
                # self._quiescent_mode = False
        else:
            command = from_frame(frame.ForwardFrame(len(pack) * 8, pack), devicetype=self.last_enabled_type)

            if isinstance(command, gear.EnableDeviceType):
                self.last_enabled_type = command.param
            elif self.last_enabled_type != 0:
                self.last_enabled_type = 0

            # if command.is_query:
            #     self._quiescent_mode = True
            if isinstance(command, device.StartQuiescentMode):
                if isinstance(command.destination, (Broadcast, BroadcastUnaddressed)):
                    self._quiescent_mode = True
            elif isinstance(command, device.StopQuiescentMode):
                if isinstance(command.destination, (Broadcast, BroadcastUnaddressed)):
                    self._quiescent_mode = False

            elif isinstance(command, gear.DTR0):
                self._dtr[0][0] = command.param
            elif isinstance(command, gear.DTR1):
                self._dtr[0][1] = command.param
            elif isinstance(command, gear.DTR2):
                self._dtr[0][2] = command.param

            elif isinstance(command, device.DTR0):
                self._dtr[1][0] = command._opcode
            elif isinstance(command, device.DTR1):
                self._dtr[1][1] = command._opcode
            elif isinstance(command, device.DTR2):
                self._dtr[1][2] = command._opcode
            elif isinstance(command, device.DTR1DTR0):
                self._dtr[1][1] = command._instance
                self._dtr[1][0] = command._opcode
            elif isinstance(command, device.DTR2DTR1):
                self._dtr[1][2] = command._instance
                self._dtr[1][1] = command._opcode

            command.on_commit()
            logging.debug('>> %s' % command)
            if command.is_commited and not command.is_query:
                self.set_external(command)
            else:
                self.last_external = command

    def set_external(self, command, *args, response=None):
        if command:
            self.external_cmd_handler.send((command, response, *args))

        self.last_external = None

    def on_dali2_event(self, response, flags=()):
        self.event_handler.send(response)

    def release_condition(self):
        self._request_event.set()

    def set_stopped(self):
        super(DaliInterface, self).set_stopped()
        if not self._request_event.is_set():
            self._request_event.set()


class BrokerAdapter(DaliInterface):
    def on_dump(self, response, *args, **kwargs):
        response, flags = response.decode().split('#')
        flags = flags.split(':') if len(flags) > 0 else []
        response = bytes.fromhex(response) if len(response) > 0 else b''

        super(BrokerAdapter, self).on_dump(response, flags=flags)

    def send(self, command: Command):
        if self._stopped is True:
            return BackwardModuleError()

        while self._quiescent_mode and not self._stopped:
            threading.Event().wait(1)

        assert isinstance(command, Command)

        if isinstance(command, gear._GearCommand) and command.device_type > 0:
            self.send(gear.EnableDeviceType(command.device_type))

        message = command.frame.pack

        data = ''.join(hex(b).replace('0x', '').rjust(2, '0') for b in message).upper()

        flags = []
        if command.is_config:
            flags += DOUBLE_SEND_FLAG
        if command.is_query:
            flags += FORCE_ANSWER_FLAG

        if len(flags) > 0:
            data += '#' + ':'.join(flags)

        command.on_send()
        self._last_command = command
        self._response = None

        self._mqtt_client.publish(self._send_topic, data)
        logging.debug(u"-> {}".format(command))

        self._request_event.wait(self.timeout)
        if self._last_command and self._last_command.is_query and self._response is None:
            self._response = command.response(frame.BackwardModuleError())
            logging.warning(f'      <- NOT ANSWERED {self._last_command}')
        self._last_command = None
        self._request_event.clear()

        if (isinstance(self._response, BackwardModuleError) or self._response is None or self._response._value is None) and command.need_retry:
            logging.debug(f'RESEND {command}')
            self._response = self.send(command)

        return self._response


class DaliCanAdapter(DaliInterface, threading.Thread):
    cmd = False
    cl = 1
    CONTROLLER_ID = 31

    def __init__(self, *args, **kwargs):
        super(DaliCanAdapter, self).__init__(None, *args, **kwargs)
        threading.Thread.__init__(self, name=self.__class__.__name__)
        self._bus = None
        self.can_name = None
        self.module = None
        self.channel = None
        self.listener = None

    def set_stopped(self):
        super(DaliCanAdapter, self).set_stopped()
        self.bus.shutdown()

    def set_bus_params(self, send_topic):
        self.can_name, self.module, self.channel = (int(i) for i in send_topic.split('/') if i.isdigit())
        self.can_name = ('can%s' if self.can_name == 0 else 'rcan%s') % self.can_name

    @property
    def bus(self) -> can.Bus:
        if self.module is None or self.channel is None or self.can_name is None:
            raise NotImplementedError()
        if self._bus is None:
            self._bus = can.Bus(
                channel=self.can_name,
                **dict(
                    bustype='socketcan',
                    can_filters=[
                        {
                            "can_id": (self.module << 5) + self.CONTROLLER_ID,
                            "can_mask": 1023,
                        },
                    ]
                )
            )
        return self._bus

    def start(self):
        threading.Thread.start(self)

    def run(self):
        while self._stopped is False:
            try:
                logging.info('Start listening %s' % self.can_name)
                for msg in self.bus:
                    try:
                        if msg.data[0] == 0x01 and (msg.data[1] & 0x07) + 1 == self.channel:
                            self.on_dump(msg.data[2:], flags='E' if (msg.data[1] >> 5) & 1 == 1 else '')
                    except BaseException as ex:
                        logging.exception(ex)
                    finally:
                        if self._stopped:
                            break
            except (BaseException, KeyboardInterrupt) as ex:
                logging.info('Stop listening %s. %s' % (self.can_name, str(ex)))
                if isinstance(ex, KeyboardInterrupt):
                    self._stopped = True

    def send(self, command: Command):
        while self._quiescent_mode:
            threading.Event().wait(1)

        if isinstance(command, gear._GearCommand) and command.device_type > 0:
            self.send(gear.EnableDeviceType(command.device_type))

        command.on_send()
        self._last_command = command
        self._response = None

        self.bus.send(can.Message(
            arbitration_id=(DaliCanAdapter.CONTROLLER_ID << 5) + self.module,
            data=bytes((
                (1 << 7 if self.cmd else 0) + self.cl,
                sum((
                    1 << 0 if command.is_config else 0,
                    1 << 1 if len(command.frame) == 24 else 0,
                    1 << 2 if command.is_query else 0
                )),
                1 << self.channel - 1
            )) + command.frame.pack,
            extended_id=False
        ))
        logging.debug(u"-> {}".format(command))

        self._request_event.wait(self.timeout)
        if self._last_command and self._last_command.is_query and self._response is None:
            self._response = command.response(frame.BackwardModuleError())
            logging.warning(f'      <- NOT ANSWERED {self._last_command}')
        self._last_command = None
        self._request_event.clear()

        if (isinstance(self._response, BackwardModuleError) or self._response is None or self._response._value is None) and command.need_retry:
            logging.debug(f'RESEND {command}')
            self._response = self.send(command)

        return self._response


def str_bytes(_bytes: bytes): return ' '.join(hex(b)[2:].rjust(2, '0') for b in _bytes)

==> spread-5.3.0.5-py3-none-any.whl: spread_core/protocols/dali/driver/dali_interface.py <==
import logging
import threading

import can

from .. import frame
from ..address import Broadcast, BroadcastUnaddressed
from ..command import Command, YesNoResponse, from_frame
from ..device import general as device
from ..gear import general as gear
from ....tools.base_interface import AbstractInterfaceClass

_can_logger = logging.getLogger('can.interfaces.socketcan.socketcan')
_can_logger.level = logging.WARNING
_can_logger.getChild("rx").level = logging.WARNING

DOUBLE_SEND_FLAG = 'D'
THREE_BYTES_FLAG = 'F'
FORCE_ANSWER_FLAG = 'W'
DALI_ERROR_FLAG = 'E'


class DaliInterface(AbstractInterfaceClass):
    wait_for_ready = True

    _response = None
    _request_event = threading.Event()
    _last_command = None
    last_external = None
    last_enabled_type = 0
    _dtr = [[0, 0, 0], [0, 0, 0]]
    _quiescent_mode = False

    def dtr(self, command, dtr_id):
        dali_index = 0 if isinstance(command, gear._GearCommand) else 1
        return self._dtr[dali_index][dtr_id]

    def _send_dali(self, command: Command):
        raise NotImplementedError()

    def send(self, command: Command):
        assert isinstance(command, Command)

        while not self._stopped and (not self.ready or self._quiescent_mode):
            threading.Event().wait(1)

        if self._stopped:
            return None

        self._send_dali(command)

        logging.debug("-> %s" % command)

        has_answered = self._request_event.wait(self.timeout)

        if not has_answered:
            logging.warning(f'      <- NOT ANSWERED {command}')
            if command.need_retry:
                self._response = self.send(command)
        self._request_event.clear()

        self._last_command = None

        return self._response

    def on_dump(self, response, *args, **kwargs):
        command = self._last_command

        if DALI_ERROR_FLAG in kwargs.get('flags', []):
            logging.debug('Error flag received with frame "%s"' % str_bytes(response))
        elif len(response) == 3 and response[0] & 1 == 0:
            self.on_dali2_event(response)
        elif command and response == command.frame.pack:
            command.on_commit()
            logging.debug('  -> ✓: {}'.format(str_bytes(response)))
            if command.is_commited and not command.is_query:
                self.release_condition()
        elif command and len(response) <= 1:
            command.on_response()
            try:
                if response:
                    result = command.response(frame.BackwardFrame(response))
                elif command.response._expected or command.response == YesNoResponse:
                    result = command.response(None)
                else:
                    result = None

                self._response = result

                _flags_str = 'flags: [{}]'.format(', '.join(kwargs['flags'])) if kwargs.get('flags', []) else ''
                logging.debug('    <- {}'.format('; '.join([str(result) if response else '<empty>', _flags_str])))
            except BaseException as ex:
                logging.error('[{}]: {}'.format(response, str(ex)))
            finally:
                self.release_condition()
        else:
            self.on_unknown_pack(response, *args, **kwargs)

    def on_unknown_pack(self, pack, *args, **kwargs):
        command = self.last_external
        if DALI_ERROR_FLAG in kwargs.get('flags', []):
            logging.debug('Error flag received with frame "%s"' % str_bytes(pack))
        elif command and pack == command.frame.pack:
            command.on_commit()
            logging.debug('  >> ✓: {}'.format(str_bytes(pack)))
            if command.is_commited and not command.is_query:
                self.set_external(command)
        elif command and len(pack) <= 1:
            command.on_response()
            try:
                if pack:
                    result = command.response(frame.BackwardFrame(pack))
                elif command.response._expected or command.response == YesNoResponse:
                    result = command.response(None)
                else:
                    result = None

                logging.debug('    << {}'.format('; '.join([str(result)])))

                args = []
                if isinstance(command, (gear._StandardCommand, gear._SpecialCommand)) and command._hasparam:
                    args.append(command.param)
                if command._uses_dtr0:
                    args.append(self.dtr(command, 0))
                if command._uses_dtr1:
                    args.append(self.dtr(command, 1))
                if command._uses_dtr2:
                    args.append(self.dtr(command, 2))

                self.set_external(command, *args, response=result)
            except BaseException as ex:
                logging.error('[{}]: {}'.format(pack, str(ex)))
            finally:
                self.set_external(None)
                # self._quiescent_mode = False
        else:
            command = from_frame(frame.ForwardFrame(len(pack) * 8, pack), devicetype=self.last_enabled_type)

            if isinstance(command, gear.EnableDeviceType):
                self.last_enabled_type = command.param
            elif self.last_enabled_type != 0:
                self.last_enabled_type = 0

            if isinstance(command, device.StartQuiescentMode):
                if isinstance(command.destination, (Broadcast, BroadcastUnaddressed)):
                    self._quiescent_mode = True
            elif isinstance(command, device.StopQuiescentMode):
                if isinstance(command.destination, (Broadcast, BroadcastUnaddressed)):
                    self._quiescent_mode = False

            elif isinstance(command, gear.DTR0):
                self._dtr[0][0] = command.param
            elif isinstance(command, gear.DTR1):
                self._dtr[0][1] = command.param
            elif isinstance(command, gear.DTR2):
                self._dtr[0][2] = command.param

            elif isinstance(command, device.DTR0):
                self._dtr[1][0] = command._opcode
            elif isinstance(command, device.DTR1):
                self._dtr[1][1] = command._opcode
            elif isinstance(command, device.DTR2):
                self._dtr[1][2] = command._opcode
            elif isinstance(command, device.DTR1DTR0):
                self._dtr[1][1] = command._instance
                self._dtr[1][0] = command._opcode
            elif isinstance(command, device.DTR2DTR1):
                self._dtr[1][2] = command._instance
                self._dtr[1][1] = command._opcode

            command.on_commit()
            logging.debug('>> %s' % command)
            if command.is_commited and not command.is_query:
                self.set_external(command)
            else:
                self.last_external = command

    def set_external(self, command, *args, response=None):
        if command:
            self.external_cmd_handler.send((command, response, *args))

        self.last_external = None

    def on_dali2_event(self, response, flags=()):
        self.event_handler.send(response)

    def release_condition(self):
        self._request_event.set()

    def set_stopped(self):
        super(DaliInterface, self).set_stopped()
        if not self._request_event.is_set():
            self._request_event.set()


class BrokerAdapter(DaliInterface):
    def on_dump(self, response, *args, **kwargs):
        response, flags = response.decode().split('#')
        flags = flags.split(':') if len(flags) > 0 else []
        response = bytes.fromhex(response) if len(response) > 0 else b''

        super(BrokerAdapter, self).on_dump(response, flags=flags)

    def _send_dali(self, command: Command):
        assert isinstance(command, Command)

        if isinstance(command, gear._GearCommand) and command.device_type > 0:
            self.send(gear.EnableDeviceType(command.device_type))

        data = ''.join(hex(b)[2:].rjust(2, '0').upper() for b in command.frame.pack)

        flags = []
        if command.is_config:
            flags += DOUBLE_SEND_FLAG
        if command.is_query:
            flags += FORCE_ANSWER_FLAG

        if len(flags) > 0:
            data += '#' + ':'.join(flags)

        command.on_send()
        self._last_command = command
        self._response = None

        self._mqtt_client.publish(self._send_topic, data)


class DaliCanAdapter(DaliInterface, threading.Thread):
    SERVICE_CMD = 1
    PROTOCOL_CMD = 0
    cl = 1
    CONTROLLER_ID = 31

    def __init__(self, *args, **kwargs):
        super(DaliCanAdapter, self).__init__(None, *args, **kwargs)
        threading.Thread.__init__(self, name=self.__class__.__name__)
        self._bus = None
        self.can_name = None
        self.module = None
        self.channel = None
        self.listener = None

    def set_stopped(self):
        super(DaliCanAdapter, self).set_stopped()
        self.bus.shutdown()

    def set_bus_params(self, send_topic):
        self.can_name, self.module, self.channel = (int(i) for i in send_topic.split('/') if i.isdigit())
        self.can_name = ('can%s' if self.can_name == 0 else 'rcan%s') % self.can_name

    @property
    def bus(self) -> can.Bus:
        if self.module is None or self.channel is None or self.can_name is None:
            raise NotImplementedError()
        if self._bus is None:
            self._bus = can.Bus(
                channel=self.can_name,
                **dict(
                    bustype='socketcan',
                    can_filters=[
                        {
                            "can_id": (self.module << 5) + self.CONTROLLER_ID,
                            "can_mask": 1023,
                        },
                    ]
                )
            )
        return self._bus

    def start(self):
        threading.Thread.start(self)

    def run(self):
        from ....protocols.can_utils.rapida.responses import LineStateEvent, LineStatusResponse
        from ....protocols.can_utils.rapida.commands import QueryLineStatus

        self.bus.send(can.Message(
            arbitration_id=(DaliCanAdapter.CONTROLLER_ID << 5) + self.module,
            data=bytes(((self.SERVICE_CMD << 7) + self.cl,)) + QueryLineStatus().frame,
            extended_id=False
        ))

        while self._stopped is False:
            try:
                logging.info('Start listening %s' % self.can_name)
                for msg in self.bus:
                    if msg.data:
                        try:
                            if len(msg.data) > 1 and msg.data[0] == 0x01:
                                if (msg.data[1] & 0x07) + 1 == self.channel:
                                    self.on_dump(msg.data[2:], flags=DALI_ERROR_FLAG if (msg.data[1] >> 5) & 1 else '')
                            elif len(msg) > 2 and msg.data[0] == 0x81:
                                if msg.data[1] == LineStateEvent.opcode:
                                    if msg.data[2] == self.channel-1:
                                        ready = msg.data[3]
                                        self.on_status(ready)
                                elif msg.data[1] == LineStatusResponse.opcode:
                                    ready = bool((2 ** (self.channel - 1)) & msg.data[2])
                                    self.on_status(ready)
                        except BaseException as ex:
                            logging.exception(ex)
                        finally:
                            if self._stopped:
                                break
            except (BaseException, KeyboardInterrupt) as ex:
                logging.info('Stop listening %s. %s' % (self.can_name, str(ex)))
                if isinstance(ex, KeyboardInterrupt):
                    self._stopped = True

    def _send_dali(self, command: Command):
        if isinstance(command, gear._GearCommand) and command.device_type > 0:
            self.send(gear.EnableDeviceType(command.device_type))

        self._last_command = command

        command.on_send()
        self._last_command = command
        self._response = None

        self.bus.send(can.Message(
            arbitration_id=(DaliCanAdapter.CONTROLLER_ID << 5) + self.module,
            data=bytes((
                self.cl,
                sum((
                    1 << 0 if command.is_config else 0,
                    1 << 1 if len(command.frame) == 24 else 0,
                    1 << 2 if command.is_query else 0
                )),
                1 << self.channel - 1
            )) + command.frame.pack,
            extended_id=False
        ))


def str_bytes(_bytes: bytes): return ' '.join(hex(b)[2:].rjust(2, '0') for b in _bytes)
